# -*- coding: utf8 -*-
from __future__ import unicode_literals

import fdb
import time

from driverng.transport.base import BaseTransport


class QCMLTransport(BaseTransport):
    def __init__(self, *args, **kwargs):
        self.qcml_kwargs = {"charset": "UNICODE_FSS"}
        for key, name in (
            ("hostname", "host"),
            ("database", "database"),
            ("port", "port"),
            ("username", "user"),
            ("password", "password"),
        ):
            self.qcml_kwargs[name] = kwargs.pop(key)

        super(QCMLTransport, self).__init__(**kwargs)
        self.con = None
        self.nb_poll = 0

    def fileno(self):
        return None

    def set_timeout(self, timeout):
        pass

    def _read(self, size):
        pass

    def reset(self):
        self.close()
        self.open()

    def open(self):
        if self.con:
            return
        try:
            self.con = fdb.connect(**self.qcml_kwargs)
        except fdb.OperationalError:
            time.sleep(30)
            raise

    def close(self):
        if self.con:
            self.con.close()
        self.con = None

    def poll(self, timeout, wqueue):
        if self.nb_poll >= 5:
            self.close()
            self.nb_poll = 0
        self.nb_poll += 1
        time.sleep(1)
        return False

    def _write(self, lines):
        datas = []
        self.open()
        cursor = self.con.cursor()
        try:
            for line in lines.split(b"\n"):
                self.logger.info("send VQAT_QC_IN:<%s>", line)
                cursor.callproc("VQAT_QC_IN", (line.replace(b"\r", b""),))
                qcml_id = cursor.fetchone()[0]
                cursor.connection.commit()
                while True:
                    # We recommand you to wait 300 ms before interrogating
                    # VQAT_QC_STATE to know the state of the test
                    time.sleep(0.300)
                    cursor.callproc("VQAT_STATE")
                    state = cursor.fetchone()[0]
                    self.logger.info("VQAT_STATE => %s" % state)
                    # 0 => OK
                    # 1 => ERROR
                    # 2 => WAIT
                    if state == 1:
                        raise Exception("QCML is stopped")
                    if state == 0:
                        break
                cursor.callproc("VQAT_QC_STATE", (qcml_id,))
                data = cursor.fetchone()[0]
                self.logger.info("VQAT_QC_STATE(%s) => '%s', '%s'", qcml_id, line, data)
                if data:
                    datas.append(data)
        finally:
            cursor.close()
        return b"\n".join(datas)
